// -----------------------------------------------------------------------------------------------------------------
// Documentation of showHide.js
// -----------------------------------------------------------------------------------------------------------------

/*
This javascript is used to show or hide substeps of the Canoo Webtest 'group' steptype in a HTML Webtest Resultfile.

The script provides functions to show or hide substeps for different levels in the hierarchically structured html tree:

	1) For a single group step: 	showSubstepsOfGroup / hideSubstepsOfGroup / changeLevelOfDetailForGroup
	2) For a testspec: 		showAllSubstepsOfTestspec / hideAllSubstepsOfTestspec
	3) For the entire document: 	hideAllSubstepsOnPage

The function's algorithm is as follows:

 1)   Functions for a single group step:

      The functions for a single group step get as argument the node which represents
      the image that has been clicked in the browser to show or hide the substeps of a group step.
      These functions navigate in the DOM tree to the node which is the parent of all substeps of the group.
      For each substep the attribute 'style.display' is assigned the appropriate value to show or hide the substep.

 2/3) Functions for a testspec or the entire document:

      These functions get as argument the node which represents the image that has been clicked
      in the browser to show or hide the substeps of the group steps within a tespec or within the entire
      document. They determine each image in the scope of the testpec respectively in the document
      that is used to show or hide the substeps of a single group step and use each of these images
      to call the according function for a single group step to show or hide the substeps for each
      of the group steps.

The functions are based on the following structure of the HTML document:

 1)   showSubstepsOfGroup/hideSubstepsOfGroup:

		<tr> <!-- tablerow for one step in a testspec -->
			<td> 	<!-- (( 2 )) SKRIPT: td node = imageForGroup.parentNode -->
				<!-- position of step in testspec -->
					<b>1</b>
					<br>
					<img onclick="changeLevelOfDetailForGroup(this)" name="collapseButton" src="./images/expandall.png">
					<!-- (( 1 )) SKRIPT: img node = imageForGroup -->
			</td>
			<td>	<!-- result of step execution (failed, successful, not executed) -->
				<img src="./images/ok.gif">
			</td>
			<td>	<!-- stepType and stepID -->
				<b>group</b>
				<br>
				stepID here and optional lastResult
			</td>
			<td> <!-- (( 3 )) SKRIPT: td node = tdContainingImage.nextSibling.nextSibling.nextSibling -->
				<table>
					<tr><th>No</th><th>Result</th><th>Name</th><th>Parameter</th><th>Duration</th></tr>
					<tr> ... substeps of group step ... </tr>

 2)   showAllSubstepsOfTestspec/hideAllSubstepsOfTestspec:

			<p>  <!-- (( 2 )) SKRIPT: p node: pContainingImage = imageForTestSpec.parentNode
				<img alt="show all substeps of 'group' steps in testspec" src="./images/expandall.png" onclick="showAllSubstepsOfTestspec(this)">
				<img alt="hide all substeps of 'group' steps in testspec" src="./images/collapseall.png" onclick="hideAllSubstepsOfTestspec(this)">
				<!-- (( 1 )) SKRIPT: img node = imageForTestSpec -->
			</p>

			<table> <!-- (( 3 )) SKRIPT: table node =  pContainingImage.nextSibling -->
				<tr><th>No</th><th>Result</th><th>Name</th><th>Parameter</th><th>Duration</th></tr>
				<tr> ... steps of testspec ... <tr>
*/

var fExpandSource, fCollapseSource;

/**
 * Function must be called early, such as 'onLoad', to set the two global variables with the two parameters.
 * @param expandSourceVar name of the icon file to show when the group is closed.
 * @param collapseSourceVar name of the icon file to show when the group is open.
 */
function hideAllSubstepsOnPage(expandSourceVar, collapseSourceVar)
	{
	fExpandSource = expandSourceVar;
	fCollapseSource = collapseSourceVar;
	hideAllSubsteps(document);
	}

function hideAllSubstepsOfTestspec(imageForTestSpec)
	{
	hideAllSubsteps(locateTable(imageForTestSpec));
	}

function hideAllSubsteps(nodeContainingStepsOfTestspec)
{
    var images = nodeContainingStepsOfTestspec.getElementsByTagName("img");
    for (var i = 0; i < images.length; i++)
    {
        if (images[i].getAttribute("name") == "collapseButton")
        {
            hideSubstepsOfGroup(images[i]);
        }
    }
}

function showAllSubstepsOfTestspec(imageForTestSpec)
    {
	showAllSubsteps(locateTable(imageForTestSpec));
    }

function showAllSubsteps(nodeContainingStepsOfTestspec)
	{
	var images = nodeContainingStepsOfTestspec.getElementsByTagName("img");
	for (var i=0; i<images.length; i++)
		{
        if (images[i].getAttribute("name") == "collapseButton")
			{
			showSubstepsOfGroup(images[i]);
			}
		}
	}

function changeLevelOfDetailForGroup(imageForGroup)
	{
	if (imageForGroup.src.indexOf(fCollapseSource) > -1)
		{
		hideSubstepsOfGroup(imageForGroup);
		}
	else if (imageForGroup.src.indexOf(fExpandSource) > -1)
		{
		showSubstepsOfGroup(imageForGroup);
		}
	}

function hideSubstepsOfGroup(imageForGroup)
{
    changeSubstepsOfGroup(imageForGroup, "collapsed", fExpandSource);
}

function showSubstepsOfGroup(imageForGroup)
{
    changeSubstepsOfGroup(imageForGroup, "expanded", fCollapseSource);
}

function changeSubstepsOfGroup(imageForGroup, displayStyle, imageName)
	{
    // how to locate table[@class="expanded"] or table[@class="collapsed]?
	var tdContainingImage = imageForGroup.parentNode;
	var tdContainingSubsteps = tdContainingImage.nextSibling.nextSibling.nextSibling;
	var substeps = tdContainingSubsteps.childNodes;
	for (var j = 1; j < substeps.length; j++)
		{
		// Note: nodeType==1 denotes Element; see e.g. <http://www.zytrax.com/tech/dom/nodetype.html>
		if (substeps[j].nodeType == 1)
			{
			substeps[j].className = displayStyle;
			}
		}
	imageForGroup.src = imageForGroup.src.substr(0, imageForGroup.src.lastIndexOf("/")+1)+imageName;
	}

function locateTable(imageForTestSpec)
{
    var pContainingStepsOfTestspec = imageForTestSpec.parentNode;
    do
    {
        pContainingStepsOfTestspec = pContainingStepsOfTestspec.nextSibling;
        // Note: nodeType==1 denotes Elements; see e.g. <http://www.zytrax.com/tech/dom/nodetype.html>
    }
    while (pContainingStepsOfTestspec.nodeType != 1);
    return pContainingStepsOfTestspec;
}